"""Simple Machine Learning Tool Kit package

This package contains the modules to simplify your code for your machine learning processes.

It is part of the educational repositories (https://github.com/pandle/materials)
to learn how to write stardard code and common uses of the TDD.

Package contents two classes to manage data preprocessing, testing and reporting.

    >>> import smltk
    >>> help(smltk)
    >>> import smltk.metrics as Metrics
    >>> help(Metrics)

# license MIT
# support https://github.com/bilardi/smltk/issues
"""
__version__ = '0.0.3'
__author__ = 'Alessandra Bilardi'
